from rptk import _BaseObject
from rptk.configuration import Config


class Dispatcher(_BaseObject):
    def __init__(self, config=None):
        super(Dispatcher, self).__init__()
        self.log_init()
        if not isinstance(config, Config):
            raise TypeError("%s not of type %s" % (config, Config))
        self.log.debug("initialising with config object %s" % config)
        self._config = config
        self.log_init_done()

    @property
    def config(self):
        return self._config

    @property
    def log(self):
        return self._log

    def dispatch(self, obj=None, name=None, test=False):
        self.log_method_enter(method=self.current_method)
        if obj and not name:
            self.log.debug(msg="name not provided using object (%s)" % obj)
            name = obj
        self.log.debug(msg="trying to begin query")
        try:
            with self.config.args.query_class(config=self.config) as q:
                result = q.query(obj=obj)
        except Exception as e:
            self.log.error(msg=e.message)
            raise e
        self.log.debug(msg="trying to format result for output")
        try:
            with self.config.args.format_class(config=self.config) as f:
                output = f.format(result=result, name=name)
                if test:
                    return f.validate(output=output)
        except Exception as e:
            self.log.error(msg=e.message)
            raise e
        self.log_method_exit(method=self.current_method)
        return output



import os
import ConfigParser
import argparse
import logging
from rptk import _BaseObject
from rptk.load import ClassLoader


class Config(_BaseObject):
    def __init__(self, argv=None, opts=None, parser=None):
        super(Config, self).__init__()
        self.log_init()
        if not argv:
            self.log.debug(msg="no command line args received")
            argv = list()
        if not parser:
            parser = argparse.ArgumentParser(add_help=False)
        config = ConfigParser.SafeConfigParser()
        default_config_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'rptk.conf')
        parser.add_argument(
            '--config_path', '-f', action='store', type=str,
            help="path to configuration file", default=default_config_path
        )
        partial_args, remaining_args = parser.parse_known_args(args=argv)
        self.log.debug(msg="using config file %s" % partial_args.config_path)
        config.read(partial_args.config_path)
        try:
            defaults = dict(config.items("defaults"))
        except ConfigParser.Error as e:
            self.log.error(msg=e.message)
            raise e
        if opts:
            if not isinstance(opts, dict):
                self.raise_type_error(arg=opts, cls=dict)
            self.log.debug(msg="reading opts dictionary")
            defaults.update(opts)
        self.log.debug(msg="loading query classes")
        query_class_loader = self._get_loader(items=config.items("query-classes"))
        self.log.debug(msg="loading format classes")
        format_class_loader = self._get_loader(items=config.items("format-classes"))
        parser.set_defaults(**defaults)
        parser.add_argument(
            '--query', '-Q', action='store', type=str,
            help="query class", choices=query_class_loader.class_names
        )
        parser.add_argument(
            '--format', '-F', action='store', type=str,
            help="format class", choices=format_class_loader.class_names
        )
        parser.add_argument('--host', action='store', type=str, help="irrd host to connect to")
        parser.add_argument('--port', action='store', type=int, help="irrd service tcp port")
        parser.add_argument('--name', action='store', type=str, help="prefix-list name (default: object)")
        parser.add_argument('--help', action='help', help="print usage information and exit")
        parser.add_argument('object', nargs='?', action='store', type=str, help="rpsl object name")
        self.log.debug(msg="parsing %s command-line args" % len(remaining_args))
        args = parser.parse_args(args=remaining_args)
        if not args.object:
            self.log.debug(msg="no object provided")
            args.object = None
        if not args.name:
            self.log.debug(msg="no ouput name provided")
            args.name = args.object
        args.query_class = query_class_loader.get_class(args.query)
        args.format_class = format_class_loader.get_class(args.format)
        self._args = args
        self.log_init_done()

    @property
    def args(self):
        return self._args

    @property
    def log(self):
        return self._log

    def _get_loader(self, items=None):
        return ClassLoader(items=items)



import sys
import logging
import argparse
from rptk import configuration, dispatch


def main():
    log = logging.getLogger(__name__)
    try:
        parser = argparse.ArgumentParser(add_help=False)
        parser.add_argument(
            '--debug', '-d', action='store_const', const=logging.DEBUG, dest='log_level',
            help="print debug logging output", default=logging.WARNING
        )
        args, argv = parser.parse_known_args()
        lh = logging.StreamHandler()
        lf = logging.Formatter(fmt="%(asctime)s %(name)s: %(levelname)s %(message)s")
        lh.setFormatter(lf)
        logging.getLogger().addHandler(lh)
        logging.getLogger().setLevel(args.log_level)
    except Exception as e:
        log.error(msg=e.message)
        return 1
    try:
        log.debug(msg="started")
        log.debug(msg="have %s arguments" % len(argv))
        config = configuration.Config(argv=argv, parser=parser)
        log.debug(msg="configuration done")
        dispatcher = dispatch.Dispatcher(config=config)
        log.debug(msg="dispatcher ready")
        result = dispatcher.dispatch()
        log.debug(msg="got result length %s" % len(result))
        sys.stdout.write("%s\n" % result)
        return 0
    except Exception as e:
        log.error(msg=e.message)
        return 1

if __name__ == "__main__":
    exit_code = main()
    exit(exit_code)



from rptk import configuration, dispatch


class Rptk(object):
    def __init__(self, **kwargs):
        self.opts = kwargs
        self.config = configuration.Config(opts=self.opts)
        self.dispatcher = dispatch.Dispatcher(config=self.config)

    def query(self, obj=None, name=None, test=False):
        return self.dispatcher.dispatch(obj=obj, name=name, test=test)



import os
import ConfigParser
from unittest import TestCase
from rptk import load, api


class TestRptk(TestCase):
    config_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'tests.conf')
    argv = ['--config_path', config_path]
    if os.name == 'posix':
        posix = True
    else:
        posix = False

    def test_api(self):
        cfg = ConfigParser.SafeConfigParser()
        cfg.read(self.config_path)
        query_class_loader = load.ClassLoader(items=cfg.items("query-classes"))
        format_class_loader = load.ClassLoader(items=cfg.items("format-classes"))
        obj = cfg.get(section="defaults", option="object")
        name = cfg.get(section="defaults", option="name")
        for q in query_class_loader.class_names:
            query_class = query_class_loader.get_class(name=q)
            if query_class.posix_only and not self.posix:
                continue
            for f in format_class_loader.class_names:
                opts = {'query': q, 'format': f}
                rptk = api.Rptk(**opts)
                result = rptk.query(obj=obj, name=name, test=True)
                self.assertTrue(result)
                if result:
                    print "api query with query=%s format=%s passed" % (q, f)



from rptk import _BaseObject
from rptk.configuration import Config


class BaseQuery(_BaseObject):
    posix_only = False

    def __init__(self, config=None):
        super(BaseQuery, self).__init__()
        self.log_init()
        if not isinstance(config, Config):
            self.raise_type_error(arg=config, cls=Config)
        self.log.debug("initialising with config object %s" % config)
        self._config = config
        self.log_init_done()

    @property
    def config(self):
        return self._config

    def query(self, obj=None):
        self.log_method_enter(method=self.current_method)
        if not obj:
            self.log.debug(msg="using object from configuration")
            obj = self.config.args.object
        if not isinstance(obj, basestring):
            self.raise_type_error(arg=obj, cls=basestring)
        return unicode(obj)



from time import sleep
from rptk.api import Rptk
from logging import FileHandler
try:
    from daemonize import Daemonize
    can_daemonize = True
except ImportError:
    can_daemonize = False


def init():
    if can_daemonize:
        pid_file = __name__ + ".pid"
        daemon = Daemonize(app=__name__, action=main, pid=pid_file)
        daemon.start()
    else:
        main()


def main():
    log_file = __name__ + ".log"
    logging_handler = FileHandler(log_file)
    rptk = Rptk(format="ios")
    while True:
        print rptk.query(obj="AS37271")
        sleep(3)
    return

if __name__ == "__main__":
    init()
